const { refObject, ImageButton, TextBox, Vector } = require('@tabletop-playground/api');
const { SetIdObject, TypeCharacteristic } = require('./general/General_Functions.js');
//-----------------------------------------------------------------
refObject.onCreated.add(() => {
  SetIdObject(refObject.getName(), refObject.getId());
  loadState();
})
//-----------------------------------------------------------------
const zPosition = 0.1;
const widgetWidth = 1600;
const widgetHeight = 1000;

let countSkills = 19;
let majorSkills = [];
let baffSkills = [], baffValues = [];
let debaffSkills = [], debaffValues = [];
let mainSkills = [], mainValues = [];
let startValueMainSkills = [];
let additionValueMain = [];
for (let i = 0; i < countSkills; i++) {
  majorSkills[i] = new TextBox().setText("0").setEnabled(false);
  majorSkills[i].setFontSize(40);
  majorSkills[i].onTextChanged.add(() => {
    saveState();
  })
  //--
  baffValues[i] = "0";
  baffSkills[i] = new TextBox().setText("0").setInputType(4);
  baffSkills[i].setFontSize(40);
  baffSkills[i].onTextChanged.add((_1, _2, text) => {
    baffValues[i] = text;
    RecalculationMajorSkills(i);
  })
  //--
  debaffValues[i] = "0";
  debaffSkills[i] = new TextBox().setText("0").setInputType(4);
  debaffSkills[i].setFontSize(40);
  debaffSkills[i].onTextChanged.add((_1, _2, text) => {
    debaffValues[i] = text;
    RecalculationMajorSkills(i);
  })
  //--
  mainValues[i] = "0";
  mainSkills[i] = new TextBox().setText("0").setInputType(4);
  mainSkills[i].setFontSize(40);
  mainSkills[i].onTextChanged.add((_1, _2, text) => {
    mainValues[i] = text;
    RecalculationMajorSkills(i);
    //characteristics.RecalculationFreePoints();
  })
  //--
  startValueMainSkills[i] = 0;
  additionValueMain[i] = 0;
}
//-----------------
function RecalculationMajorSkills(index) {
  let main = additionValueMain[index] + startValueMainSkills[index] + parseInt(mainSkills[index].getText());
  let debaff = parseInt(debaffSkills[index].getText());
  let baff = parseInt(baffSkills[index].getText());
  majorSkills[index].setText(main + baff - debaff);
}

function CreateChangerTextBox(parent, index, position, array, isMain) {
  let imageSize = 100;
  let buttonOffsetY = 10;

  let decrement = new ImageButton().setImage("minus.png");
  decrement.setImageSize(imageSize);
  decrement.onClicked.add(() => {
    let currentValue = array[index].getText();
    if (currentValue > 0) {
      array[index].setText(currentValue - 1);
    }
  })
  parent.addChild(decrement, position.x - 60, position.y + buttonOffsetY, 50, 50);
  //--
  parent.addChild(array[index], position.x, position.y, 80, 65);
  //--
  let increment = new ImageButton().setImage("plus.png");
  increment.setImageSize(imageSize);
  increment.onClicked.add(() => {
    let currentValue = parseInt(array[index].getText());
    array[index].setText(currentValue + 1);
  })
  parent.addChild(increment, position.x + 90, position.y + buttonOffsetY, 50, 50);
}
//-----------------------------------------------------------------
class SkillsMajor {
  constructor(parent, position) {
    this.parent = parent;
    this.showPosition = position;
    this.hidePosition = position.add(new Vector(0, 0, -zPosition));
    //-------------------------
    let nC = new Canvas();

    this.nCUI = new UIElement();
    this.nCUI.useWidgetSize = false;
    this.nCUI.position = position;
    this.nCUI.rotation = new Rotator(0, 0, 180);
    this.nCUI.widget = nC;
    this.nCUI.width = widgetWidth;
    this.nCUI.height = widgetHeight;
    this.nCUI.scale = 0.1;
    parent.attachUI(this.nCUI);
    //-------------------------
    let offsetY = 0; let offsetX = 440;
    for (let i = 0; i < countSkills; i++, offsetY++) {
      nC.addChild(majorSkills[i], offsetX, 150 + offsetY * 70, 80, 70);
      CreateChangerTextBox(nC, i, position.add(new Vector(offsetX + 180, 150 + offsetY * 70, 0)), mainSkills, true);
      if (i == 9) { offsetY = -1; offsetX = 1170; }
    }
    //-------------------------
    let borderMain = new Border();
    borderMain.setColor(new Color(0.05, 1, 0.05));
    nC.addChild(borderMain, 640, 100, 40, 40);
    nC.addChild(borderMain, 1370, 100, 40, 40);
    //-------------------------
    let arrow = new ImageButton().setImage("strelka.png");
    arrow.setImageSize(50);
    nC.addChild(arrow, 1525, 25, 50, 50);
    //-------------------------
    let t = this;
    arrow.onClicked.add(function () {
      t.HideUI();
      skillsChanger.ShowUI();
    });
  }

  HideUI() {
    this.nCUI.position = this.hidePosition;
    this.parent.updateUI(this.nCUI);
  }
  ShowUI() {
    this.nCUI.position = this.showPosition;
    this.parent.updateUI(this.nCUI);
  }
}
let skillsMajor = new SkillsMajor(refObject, new Vector(0, 0, zPosition));

class SkillsChanger {
  constructor(parent, position) {
    this.parent = parent;
    this.showPosition = position.add(new Vector(0, 0, zPosition));
    this.hidePosition = position;
    //-------------------------
    let nC = new Canvas();

    this.nCUI = new UIElement();
    this.nCUI.useWidgetSize = false;
    this.nCUI.position = position;
    this.nCUI.rotation = new Rotator(0, 0, 180);
    this.nCUI.widget = nC;
    this.nCUI.width = widgetWidth;
    this.nCUI.height = widgetHeight;
    this.nCUI.scale = 0.1;
    parent.attachUI(this.nCUI);
    //-------------------------
    let offsetY = 0; let offsetX = 500;
    for (let i = 0; i < countSkills; i++, offsetY++) {
      CreateChangerTextBox(nC, i, position.add(new Vector(offsetX, 150 + offsetY * 70, 0)), baffSkills);
      CreateChangerTextBox(nC, i, position.add(new Vector(offsetX + 210, 150 + offsetY * 70, 0)), debaffSkills);
      if (i == 9) { offsetY = -1; offsetX = 1230; }
    }
    //-------------------------
    let borderBaff = new Border();
    borderBaff.setColor(new Color(0.05, 0, 1));
    nC.addChild(borderBaff, 520, 100, 40, 40);
    nC.addChild(borderBaff, 1250, 100, 40, 40);
    //-------------------------
    let borderDebaff = new Border();
    borderDebaff.setColor(new Color(1, 0, 0.05));
    nC.addChild(borderDebaff, 730, 100, 40, 40);
    nC.addChild(borderDebaff, 1460, 100, 40, 40);
    //-------------------------
    let arrow = new ImageButton().setImage("strelka.png");
    arrow.setImageSize(50);
    nC.addChild(arrow, 1525, 25, 50, 50);
    //-------------------------
    let t = this;
    arrow.onClicked.add(function () {
      t.HideUI();
      skillsMajor.ShowUI();
    });
  }

  HideUI() {
    this.nCUI.position = this.hidePosition;
    this.parent.updateUI(this.nCUI);
  }
  ShowUI() {
    this.nCUI.position = this.showPosition;
    this.parent.updateUI(this.nCUI);
  }
}
let skillsChanger = new SkillsChanger(refObject, new Vector(0, 0, 0));
//-----------------------------------------------------------------
function saveState() {
  let state = {};

  state["baff"] = baffValues;
  state["debaff"] = debaffValues;
  state["main"] = mainValues;

  refObject.setSavedData(JSON.stringify(state));
}

function loadState() {
  if (refObject.getSavedData() === "") {
    saveState();
    return;
  }

  let state = JSON.parse(refObject.getSavedData());
  baffValues = state["baff"] || [];
  debaffValues = state["debaff"] || [];
  mainValues = state["main"] || [];
  for (let i = 0; i < countSkills; i++) {
    baffSkills[i].setText(baffValues[i] || "0");
    debaffSkills[i].setText(debaffValues[i] || "0");
    mainSkills[i].setText(mainValues[i] || "0");
  }
}

refObject.ResetValue = function () {
  for (let i = 0; i < countSkills; i++) {
    majorSkills[i].setText("0");
    baffSkills[i].setText("0");
    debaffSkills[i].setText("0");
    mainSkills[i].setText("0");
  }
  saveState();
}

refObject.RecalculationMain = (array) => {
  startValueMainSkills[0] = 5 + array[TypeCharacteristic.dextery] * 4;
  startValueMainSkills[1] = array[TypeCharacteristic.dextery] * 2;
  startValueMainSkills[2] = array[TypeCharacteristic.dextery] * 2;
  startValueMainSkills[3] = 5 + array[TypeCharacteristic.dextery] * 4;
  startValueMainSkills[4] = 30 + (array[TypeCharacteristic.dextery] + array[TypeCharacteristic.strenght]) * 2;
  startValueMainSkills[5] = 20 + (array[TypeCharacteristic.dextery] + array[TypeCharacteristic.strenght]) * 2;
  startValueMainSkills[6] = array[TypeCharacteristic.dextery] * 4;
  startValueMainSkills[7] = (array[TypeCharacteristic.perception] + array[TypeCharacteristic.intelligence]) * 2;
  startValueMainSkills[8] = parseInt(((array[TypeCharacteristic.perception] + array[TypeCharacteristic.intelligence]) / 3)) * 5;
  startValueMainSkills[9] = (array[TypeCharacteristic.perception] + array[TypeCharacteristic.intelligence]) * 2;
  startValueMainSkills[10] = 5 + array[TypeCharacteristic.dextery] * 3;
  startValueMainSkills[11] = 10 + array[TypeCharacteristic.dextery] + array[TypeCharacteristic.perception];
  startValueMainSkills[12] = array[TypeCharacteristic.dextery] * 3;
  startValueMainSkills[13] = 10 + array[TypeCharacteristic.dextery] + array[TypeCharacteristic.perception];
  startValueMainSkills[14] = array[TypeCharacteristic.intelligence] * 4;
  startValueMainSkills[15] = (array[TypeCharacteristic.perception] + array[TypeCharacteristic.intelligence]) * 3;
  startValueMainSkills[16] = array[TypeCharacteristic.charism] * 5;
  startValueMainSkills[17] = array[TypeCharacteristic.charism] * 4;
  startValueMainSkills[18] = array[TypeCharacteristic.luck] * 5;
  for (let i = 0; i < countSkills; i++) {
    RecalculationMajorSkills(i);
  }
}

refObject.AdditionMain = (array) => {
  for (let i = 0; i < countSkills; i++) {
    if (array[i])
      additionValueMain[i] = array[i];
    else
      additionValueMain[i] = 0;
  }
  for (let i = 0; i < countSkills; i++) {
    RecalculationMajorSkills(i);
  }
}